@@ -4,28 +4,33 @@ from torch import Tensor
 from ._Loss import _Loss
 
 
-@torch.jit.script
+# @torch.jit.script
 def _binary_loss(x: torch.Tensor, y: torch.Tensor, gamma: float, p: float):
     # x.shape = y.shape = (batch, *dims)
-    # y.unique() == (0., 1.)
 
     x = torch.sigmoid(x)
 
     x, y = x.flatten(1), y.flatten(1)   # 1st dim: batch
 
-    pos, neg = x*y, x*(1-y)
+    loss = 0
+    for i in range(x.size(1)):
+        xx, yy = x[:,i], y[:,i]
+        pos = torch.masked_select(xx, yy==1)
+        neg = torch.masked_select(xx, yy==0)
 
-    diff = neg.unsqueeze(1) + gamma - pos.unsqueeze(0)
+        if pos.nelement() == 0 or neg.nelement() == 0:
+            loss += torch.tensor(0., requires_grad=True)
+            continue
+        diff = neg.unsqueeze(1) + gamma - pos.unsqueeze(0)
+        loss += torch.relu(diff).pow(p).sum()
 
-    # let gradient flows when diff>0
-    loss = torch.relu(diff).pow(p)
     return loss
 
 
 class AUCLoss(_Loss):
-    """ This loss is an approximation to WilcoxonMann-Whitney (WMW) statistic. 
-        It's used to directly maximize the Area Under the Curve (AUC) metric. 
-        Only the binary classification case is supported by this implementation. 
+    """ This loss leverages an approximation to WilcoxonMann-Whitney (WMW) statistic to directly maximize the Area Under the Curve (AUC) metric.
+
+        Please note that only the binary classification case is supported by this implementation.
 
         For more details check: https://www.aaai.org/Papers/ICML/2003/ICML03-110.pdf
 
@@ -34,10 +39,6 @@ class AUCLoss(_Loss):
                                     if not exceeded that pair doesn't contribute to the loss. 
                                     The larger the `gamma`, the more the model enlarges the gap between positive and negative predictions
         - p (Float, Default=2.0): denotes the exponent used for the aforementioned difference.
-        - reduction(String, Default='mean'): specifies the reduction to apply to the output: 'none' | 'mean' | 'sum'. 
-                                    'none': no reduction will be applied, 
-                                    'mean': the sum of the output will be divided by the number of elements in the output, 
-                                    'sum': the output will be summed.
 
     Forward Args:
         - x (Tensor): It is a (N, *dims) tensor with predictions before transforming them to probabilities.
@@ -45,8 +46,9 @@ class AUCLoss(_Loss):
 
     """
 
-    def __init__(self, gamma: float = .3, p: float = 2, reduction: str = 'mean'):
-        super(AUCLoss, self,).__init__(gamma, p, reduction)
+    def __init__(self, gamma: float = .3, p: float = 2):
+        super(AUCLoss, self,).__init__(gamma, p)
+
         if gamma < 0:
             raise ValueError(
                 f'The value of gamma={gamma} must non-negative')
@@ -58,8 +60,4 @@ class AUCLoss(_Loss):
 
     def forward(self, x: Tensor, y: Tensor):
         loss = self.loss(x, y, self.gamma, self.p)
-        if self.reduction == 'mean':
-            return loss.mean()
-        elif self.reduction == 'sum':
-            return loss.sum()
         return loss
